@@ -62,7 +62,7 @@ cor_arma <- function(formula = ~ 1, p = 0, q = 0, r = 0, cov = FALSE) {
     stop(paste("covariance formulation of ARMA structures", 
                "is only possible for effects of maximal order 1"))
   }
-  x <- list(formula = formula, p = p, q = q, r = r, cov = as.logical(cov))
+  x <- nlist(formula, p, q, r, cov = as.logical(cov))
   class(x) <- c("cor_arma", "cor_brms")
   x
 }
@@ -165,6 +165,7 @@ cor_arr <- function(formula = ~ 1, r = 1) {
 print.cor_arma <- function(x, ...) {
   cat(paste0("arma(", gsub(" ", "", Reduce(paste, deparse(x$formula))),
              ", ",get_ar(x),", ",get_ma(x),", ",get_arr(x),")"))
+  invisible(x)
 }
 
 has_arma <- function(x) {
@@ -224,89 +225,12 @@ use_cov <- function(x) {
   }
 }
 
-#' Autocorrelation Function Estimation based on MCMC-Samples (experimental)
-#' 
-#' Compute (and plot by default) the autocorrelation function based on MCMC samples
-#' 
-#' @param x A matrix, data.frame or numeric vector representing the time series. 
-#'   Each row is interpreted as a sample from an MCMC procedure, whereas the columns are taken to be points of a time series.
-#'   The colnames of x are taken to be the groups and it is assumed that observations can only correlate within each group.
-#' @param lag.max  Maximum lag at which to calculate the autocorrelation. 
-#'   Default is \code{10*log10(N/g)} where \code{N} is the number of observations and \code{g} the number of groups. 
-#'   Will be automatically limited to one less than the number of maximum observations within one group.
-#' @param plot logical. If \code{TRUE} (the default) results are plotted directly
-#' @param ... Further arguments to be passed to \code{plot.acf}.
-#'                
-#' @return An object of class \code{\link[stats:acf]{acf}}.
-#' 
-#' @examples 
-#' \dontrun{
-#' ## investigate the autorcorrelation in the residuals of a fitted model
-#' ## simulate data
-#' set.seed(123)
-#' phi <- c(0.4, 0.7, -0.3)
-#' y <- 0
-#' y[2] <- phi[1] * y[1] + rnorm(1)
-#' y[3] <- phi[1] * y[2] + phi[2] * y[1] + rnorm(1)
-#' for (i in 4:300) y[i] <- sum(phi * y[(i-1):(i-3)]) + rnorm(1)
-#' 
-#' ## fit the model
-#' fit1 <- brm(y ~ 1)
-#' summary(fit1)
-#' 
-#' ## investigate the residuals (autocorrelation clearly visible)
-#' macf(residuals(fit1, summary = FALSE), lag.max = 10)
-#' 
-#' ## fit the model again with autoregressive coefficients
-#' fit2 <- brm(y ~ 1, autocor = cor_ar(p = 3))
-#' summary(fit2)
-#' 
-#' ## investigate the residuals again (autocorrelation is gone)
-#' macf(residuals(fit2, summary = FALSE), lag.max = 10)
-#' }
-#'              
-#' @export
-macf <- function(x, lag.max = NULL, plot = TRUE, ...) {
-  series <- Reduce(paste, deparse(substitute(x)))
-  if (!is.matrix(x)) {
-    if (is.data.frame(x)) {
-      x <- as.matrix(x)
-    } else if (is.numeric(x)) {
-      x <- matrix(x, nrow = 1)
-    } else { 
-      stop("x must be a matrix, data.frame or numeric vector")
-    }
-  }
-  if (is.null(colnames(x))) {
-    group <- rep(1, ncol(x))
-  } else {
-    group <- colnames(x)
-  }
-  if (is.null(lag.max)) {
-    lag.max <- min(sort(table(group), decreasing = TRUE)[1],
-                   10 * log(ncol(x) / length(unique(group)), base = 10))
-  }
-  ac_names <- paste0("ac", 1:lag.max) 
-  lm_call <- parse(text = paste0("lm(y ~ ",paste(ac_names, collapse = "+"), 
-                                 ", data = D)"))
-  coefs <- do.call(rbind, lapply(1:nrow(x), function(i) {
-    D <- as.data.frame(arr_design_matrix(x[i, ], r = lag.max, group = group))
-    names(D) <- paste0("ac", 1:lag.max) 
-    D <- cbind(D, y = x[i, ])
-    fit <- eval(lm_call)
-    fit$coefficients[2:length(fit$coefficients)]
-  }))
-  out <- list(lag = array(0:lag.max, dim = c(lag.max + 1, 1, 1)),
-              acf = array(c(1, colMeans(coefs)), dim = c(lag.max + 1, 1, 1)), 
-              type = "correlation", 
-              n.used = ncol(x) / length(unique(group)), 
-              series = series, 
-              snames = NULL)
-  class(out) <- "acf"
-  if (plot) {
-    plot(out, ...)
-    invisible(out)
-  } else {
-    out
+check_autocor <- function(autocor) {
+  # check validity of autocor argument
+  if (is.null(autocor)) 
+    autocor <- cor_arma()
+  if (!is(autocor, "cor_brms")) { 
+    stop("autocor must be of class cor_brms")
   }
+  autocor
 }
\ No newline at end of file
