@@ -1,5 +1,5 @@
 test_that("brm produces expected errors", {
   expect_error(brm(rating~treat+period+carry+(1|subject), data = inhaler, 
-                   partial = ~treat, family = c("cratio", "logit")), 
+                   partial = ~treat, family = cratio("logit")), 
               paste("Error occured for variables: treat"))
 })
\ No newline at end of file
